import unittest

from metapub import FindIt

SAMPLE_PMIDS = { 'embargoed': [ '25575644', '25700512', '25554792', '25146281', '25766237', '25370453' ],
                 'nonembargoed': ['26098888' ],
                 'non_pmc': ['26111251', '17373727']
               }

class TestFindIt(unittest.TestCase):

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_skipping_cache(self):
        #use a known working, non-PMC pubmed ID
        src = FindIt(pmid=26111251, cachedir=None)
        assert src._cache is None
        assert src.url is not None
        assert not src.reason

    def test_using_cache(self):
        src = FindIt(pmid=SAMPLE_PMIDS['nonembargoed'][0])
        assert src.url is not None
        assert src._cache is not None

        # source from the same pmid. check that result is same as if we used no cache.
        cached_src = FindIt(pmid=SAMPLE_PMIDS['nonembargoed'][0])
        fresh_src = FindIt(pmid=SAMPLE_PMIDS['nonembargoed'][0])
        
        assert cached_src.url == fresh_src.url

    #def test_embargoed_pmid(self):
        #use a currently PMC embargoed pmid, since its status is bound to change (eventually)
    #    src = FindIt(pmid=SAMPLE_PMIDS['embargoed'][0], cachedir=None)
    #    assert src.url is None
    #    assert src.reason.startswith('DENIED')
 



from __future__ import absolute_import, print_function

__author__ = 'nthmost'

from urlparse import urlsplit

import requests
from lxml.html import HTMLParser
from lxml import etree
from ..exceptions import AccessDenied, NoPDFLink

from .journal_formats import (format_templates, sciencedirect_url, BMC_format,
                              aaas_format, aaas_journals, lancet_journals,
                              nature_format, nature_journals)

#TODO: make configurable (somehow...)
AAAS_USERNAME = 'nthmost'
AAAS_PASSWORD = '434264'


DX_DOI_URL = 'http://dx.doi.org/%s'
def the_doi_2step(doi):
    'takes a doi (string), returns a url to a paper'
    response = requests.get(DX_DOI_URL % doi)
    if response.status_code in [200, 401, 301, 302, 307, 308]:
        return response.url
    else:
        raise NoPDFLink('dx.doi.org lookup failed for doi %s (HTTP %i returned)' %
                        (doi, response.status_code))

def the_jci_polka(pma):
    '''Dance of the Journal of Clinical Investigation, which should be largely free.

         :param: pma (PubMedArticle object)
         :return: url (string)
         :raises: AccessDenied, NoPDFLink
    '''
    # approach: pii with dx.doi.org fallback to get pdf "view" page; scrape pdf download link.
    if pma.pii:
        starturl = format_templates['jci'].format(a=pma)
    elif pma.doi:
        starturl = the_doi_2step(pma.doi)
        starturl = starturl + '/pdf'
    else:
        raise NoPDFLink('MISSING: pii, doi (doi lookup failed)')

    # Iter 1: do this until we see it stop working. (Iter 2: scrape download link from page.)
    return starturl.replace('/pdf', '/version/1/pdf/render')

def the_sciencedirect_disco(pma):
    '''  :param: pma (PubMedArticle object)
         :return: url (string)
         :raises: AccessDenied, NoPDFLink
    '''
    #we're looking for a url that looks like this:
    #http://www.sciencedirect.com/science/article/pii/S0022283601953379/pdfft?md5=07db9e1b612f64ea74872842e34316a5&pid=1-s2.0-S0022283601953379-main.pdf

    starturl = None
    if pma.pii:
        starturl = sciencedirect_url.format(piit=pma.pii.translate(None, '-()'))
    elif pma.doi:
        starturl = the_doi_2step(pma.doi)

    if starturl == None:
        raise NoPDFLink('MISSING: pii, doi (doi lookup failed)')

    try:
        res = requests.get(starturl)
    except requests.exceptions.TooManyRedirects:
        raise NoPDFLink('TXERROR: ScienceDirect TooManyRedirects: cannot reach %s via %s' %
                        (pma.journal, starturl))

    tree = etree.fromstring(res.text, HTMLParser())
    try:
        div = tree.cssselect('div.icon_pdf')[0]
    except IndexError:
        raise NoPDFLink('DENIED: ScienceDirect did not provide pdf link (probably paywalled)')
    url = div.cssselect('a')[0].get('href')
    if url.find('.pdf') > -1:
        return url
    else:
        # give up, it's probably a "shopping cart" link.
        # TODO: parse return, raise more nuanced exceptions here.
        raise NoPDFLink('DENIED: ScienceDirect did not provide pdf link (probably paywalled)')

def the_biomed_calypso(pma):
    '''  :param: pma (PubMedArticle object)
         :return: url (string)
         :raises: AccessDenied, NoPDFLink
    '''
    baseid = pma.doi if pma.doi else pma.pii
    if baseid:
        article_id = baseid.split('/')[1]
    else:
        raise NoPDFLink('MISSING: doi needed for BMC article')
    return BMC_format.format(aid=article_id)


def the_aaas_tango(pma):
    '''  :param: pma (PubMedArticle object)
         :return: url (string)
         :raises: AccessDenied, NoPDFLink
    '''
    if pma.volume and pma.issue and pma.pages:
        pdfurl = aaas_format.format(ja=aaas_journals[pma.journal]['ja'], a=pma)
    elif pma.doi:
        pdfurl = the_doi_2step(pma.doi) + '.full.pdf'
    else:
        raise NoPDFLink('MISSING: doi, vip (doi lookup failed)')

    response = requests.get(pdfurl)
    if response.status_code == 200 and response.headers['content-type'].find('pdf') > -1:
        return response.url

    if response.status_code == 200 and response.headers['content-type'].find('html') > -1:
        tree = etree.fromstring(response.content, HTMLParser())
        form = tree.cssselect('form')[1]
        fbi = form.fields.get('form_build_id')

        baseurl = urlsplit(response.url)
        post_url = baseurl.scheme + '://' + baseurl.hostname + form.action

        payload = {'pass': AAAS_PASSWORD, 'name': AAAS_USERNAME,
                   'form_build_id': fbi, 'remember_me': 1}
        response = requests.post(post_url, data=payload)
        if response.status_code == 403:
            return AccessDenied('DENIED: AAAS subscription-only paper')
        elif response.headers['content-type'].find('pdf') > -1:
            return response.url
        elif response.headers['content-type'].find('html') > -1:
            raise NoPDFLink('DENIED: AAAS pdf download requires form navigation. URL: %s' % pdfurl)
    else:
        raise NoPDFLink('TXERROR: AAAS returned %s for url %s' % (response.status_code, pdfurl))


def the_jama_dance(pma):
    '''  :param: pma (PubMedArticle object)
         :return: url (string)
         :raises: AccessDenied, NoPDFLink
    '''
    url = the_doi_2step(pma.doi)
    res = requests.get(url)
    parser = HTMLParser()
    tree = etree.fromstring(res.text, parser)
    # we're looking for a meta tag like this:
    # <meta name="citation_pdf_url" content="http://archneur.jamanetwork.com/data/Journals/NEUR/13776/NOC40008.pdf" />
    for item in tree.findall('head/meta'):
        if item.get('name') == 'citation_pdf_url':
            return item.get('content')
    raise NoPDFLink('DENIED: JAMA did not provide PDF link in (%s).' % url)

def the_jstage_dive(pma):
    '''  :param: pma (PubMedArticle object)
         :return: url (string)
         :raises: AccessDenied, NoPDFLink
    '''
    url = the_doi_2step(pma.doi)
    res = requests.get(url)
    if res.url.find('jstage') > -1:
        return res.url.replace('_article', '_pdf')
    else:
        raise NoPDFLink('TXERROR: %s did not resolve to jstage article' % url)

def the_wiley_shuffle(pma):
    '''  :param: pma (PubMedArticle object)
         :return: url (string)
         :raises: AccessDenied, NoPDFLink
    '''
    res = requests.get(format_templates['wiley'].format(a=pma))
    if res.headers['content-type'].find('html') > -1:
        if res.text.find('ACCESS DENIED') > -1:
            raise AccessDenied('DENIED: Wiley E Publisher says no to %s' % res.url)

        tree = etree.fromstring(res.text, HTMLParser())
        if tree.find('head/title').text.find('Not Found') > -1:
            raise NoPDFLink('TXERROR: Wiley says File Not found (%s)' % res.url)
        iframe = tree.find('body/div/iframe')
        return iframe.get('src')

    elif res.headers['content-type'] == 'application/pdf':
        return res.url

def the_lancet_tango(pma):
    '''  :param: pma (PubMedArticle object)
         :return: url (string)
         :raises: AccessDenied, NoPDFLink
    '''
    if pma.pii:
        return format_templates['lancet'].format(a=pma, ja=lancet_journals[pma.journal.translate(None, '.')]['ja'])
    if pma.doi:
        return the_doi_2step(pma.doi).replace('abstract', 'pdf').replace('article', 'pdfs')
    else:
        raise NoPDFLink('MISSING: pii, doi (doi lookup failed)')

def the_nature_ballet(pma):
    '''  :param: pma (PubMedArticle object)
         :return: url (string)
         :raises: AccessDenied, NoPDFLink
    '''
    url = ''
    if pma.doi:
        try:
            starturl = the_doi_2step(pma.doi)
            url = starturl.replace('html', 'pdf').replace('abs', 'pdf').replace('full', 'pdf')
        except NoPDFLink:
            # alright, let's try the pii route.
            pass

    if url == '':
        if pma.pii:
            url = nature_format.format(a=pma, ja=nature_journals[pma.journal.translate(None, '.')]['ja'])
        else:
            if pma.doi:
                raise NoPDFLink('MISSING: pii, TXERROR: dx.doi.org resolution failed for doi %s' % pma.doi)
            else:
                raise NoPDFLink('MISSING: pii, doi')

    res = requests.get(url)
    if res.headers['content-type'].find('pdf') > -1:
        return res.url
    elif res.headers['content-type'].find('html') > -1:
        raise AccessDenied('DENIED: Nature denied access to %s' % res.url)
    raise NoPDFLink('TXERROR: Nature -- unknown problem retrieving from %s' % res.url)

PMC_PDF_URL = 'http://www.ncbi.nlm.nih.gov/pmc/articles/pmid/{a.pmid}/pdf'
EUROPEPMC_PDF_URL = 'http://europepmc.org/backend/ptpmcrender.fcgi?accid=PMC{a.pmc}&blobtype=pdf'
def the_pmc_twist(pma, use_nih=True):
    '''  :param: pma (PubMedArticle object)
         :return: url
         :raises: NoPDFLink
    '''
    #TODO: reevaluate when FindIt has a cache function
    if pma.history.get('pmc-release', None):
        raise NoPDFLink('DENIED: pmc article in embargo until %s' % pma.history['pmc-release'].strftime('%Y-%m-%d'))

    url = EUROPEPMC_PDF_URL.format(a=pma)
    res = requests.get(url)
    if res.headers['content-type'].find('html') > -1:
        url = PMC_PDF_URL.format(a=pma)
        if use_nih:
            # try the other PMC.
            #   URL block might be discerned by grepping for this:
            #
            #   <div class="el-exception-reason">Bulk downloading of content by IP address [162.217...,</div>
            res = requests.get(url)
            if res.headers['content-type'].find('html') > -1:
                raise NoPDFLink('TXERROR: could not get PDF url from either NIH or EuropePMC.org')
        else:
            raise NoPDFLink('TXERROR: could not get PDF from EuropePMC.org and USE_NIH set to False')

    if res.headers['content-type'].find('pdf') > -1:
        return url

    raise NoPDFLink('TXERROR: PMC download returned weird content-type %s' % res.headers['content-type'])


def the_springer_shag(pma):
    '''  :param: pma (PubMedArticle object)
         :return: url
         :raises: NoPDFLink
    '''
    # start: http://link.springer.com/article/10.1007%2Fs13238-015-0153-5
    # return: http://link.springer.com/content/pdf/10.1007%2Fs13238-015-0153-5.pdf
    if pma.doi:
        baseurl = the_doi_2step(pma.doi)
    else:
        raise NoPDFLink('MISSING: doi (doi lookup failed)')
    url = baseurl.replace('article', 'content/pdf') + '.pdf'
    res = requests.get(url)
    if not res.ok:
        raise NoPDFLink('TXERROR: %i status returned from Springer url (%s)' % (res.status_code, url))
    if res.headers['content-type'].find('pdf') > -1:
        return url
    else:
        raise AccessDenied('DENIED: Springer url (%s) resulted in HTTP %i' % (url, res.status_code))

def the_karger_conga(pma):
    '''  :param: pma (PubMedArticle object)
         :return: url
         :raises: AccessDenied, NoPDFLink
    '''
    # example: 23970213.  doi = 10.1159/000351538
    #       http://www.karger.com/Article/FullText/351538
    #       http://www.karger.com/Article/Pdf/351538
    if pma.doi:
        baseurl = the_doi_2step(pma.doi)
    else:
        raise NoPDFLink('MISSING: doi (doi lookup failed)')
    # if it directs to an "Abstract", we prolly can't get the PDF. Try anyway.
    url = baseurl.replace('FullText', 'Pdf').replace('Abstract', 'Pdf')
    res = requests.get(url)
    if not res.ok:
        raise NoPDFLink('TXERROR: %i status returned from Karger url (%s)' % (res.status_code, url))
    if res.headers['content-type'].find('pdf') > -1:
        return url
    else:
        raise AccessDenied('DENIED: Karger url (%s) did not result in a PDF' % url)




import unittest, os

from metapub import PubMedFetcher 
from metapub.pubmedfetcher import parse_related_pmids_result
from metapub.pubmedcentral import *


TEST_CACHEDIR = 'tests/cachedir'
try:
    for item in os.listdir(TEST_CACHEDIR):
        os.unlink(os.path.join(TEST_CACHEDIR, item))
except OSError:
    pass
try:
    os.rmdir(TEST_CACHEDIR)
except OSError:
    pass

class TestPubmedFetcher(unittest.TestCase):

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_article_by_pmid(self):
        pmid = '4'
        fetch = PubMedFetcher()
        article = fetch.article_by_pmid(pmid)
        assert str(article.pmid) == pmid

        pmid = '25763451'
        fetch = PubMedFetcher()
        article = fetch.article_by_pmid(pmid)
        assert str(article.pmid) == pmid

    def test_related_pmids(self):
        ''' * pubmed    (all related links)
            * citedin   (papers that cited this paper)
            * five      (the "five" that pubmed displays as the top related results)
            * reviews   (review papers that cite this paper)
            * combined  (?)
        '''

        expected_keys = ['pubmed', 'citedin', 'five', 'reviews', 'combined']
        xmlstr = open('tests/data/sample_related_pmids_result.xml').read()
        resd = parse_related_pmids_result(xmlstr)
        for key in resd.keys():
            assert key in expected_keys
        assert len(resd['citedin']) == 6

    def test_configurable_cachedir(self):
        '''test that `cachedir` keyword argument is fully supported in modes:

        cachedir == 'default'   <-- assumed working since other tests use this.
        cachedir is None
        cachedir is 'some/path'
        cachedir is '~/path'
        '''

        cachedir = TEST_CACHEDIR
        # start with cachedir==None; test that no cachedir is created.
        fetch = PubMedFetcher(cachedir=None)
        assert not os.path.exists(cachedir)

        fetch = PubMedFetcher(cachedir=cachedir)
        assert os.path.exists(cachedir)

        os.unlink(fetch._cache_path)
        os.rmdir(cachedir)

        fetch = PubMedFetcher(cachedir='~/testcachedir')
        assert os.path.exists(os.path.expanduser('~/testcachedir'))

        os.unlink(fetch._cache_path)
        os.rmdir(os.path.expanduser('~/testcachedir'))

        




from lxml.etree import XMLSyntaxError

class MetaPubError(Exception):
    pass

class InvalidPMID(MetaPubError):
    '''Raised when NCBI efetch of a pubmed ID results in "invalid" response.'''

class CrossRefConnectionError(MetaPubError):
    '''Raised when a well-formed CrossRef query results in a server error.'''

class NoPDFLink(MetaPubError):
    '''Raised when a FindIt url lookup fails for some specific reason that is
    particular to the journal or publisher.

    This Exception provides extended attributes:

            reason      :    human-readable "reason" why URL lookup failed.
            url         :    last url attempted 
            status_code :    last HTTP code returned in attempt (if any)
            missing     :    list of data items missing from last attempt (if any)

    This Exception is mostly used internally in FindIt as flow control.
    '''
    def __init__(self, reason, *args, **kwargs):
        self.message = reason
        self.reason = reason
        self.url = kwargs.get('url', None)
        self.missing = kwargs.get('missing', [])
        self.status_code = kwargs.get('status_code', None)

        super(NoPDFLink, self).__init__(reason, *args, **kwargs) 

class AccessDenied(NoPDFLink):
    '''Raised when a FindIt url lookup fails for some specific reason that is
    particular to the journal or publisher.'''





from __future__ import print_function, absolute_import

__doc__='mildly-experimental mashups of various services to get needed IDs.'

from .pubmedfetcher import PubMedFetcher
from .crossref import CrossRef
from .text_mining import re_doi
from .exceptions import *

crossref = None   #CrossRef()
pm_fetch = None   #PubMedFetcher()

def _start_engines():
    global crossref
    global pm_fetch
    if crossref and pm_fetch:
        return
    else:
        crossref = CrossRef()
        pm_fetch = PubMedFetcher()

def _protected_crossref_query(**kwargs):
    pma = kwargs.get('pma', None)
    pmid = kwargs.get('pmid', None)
    use_best_guess = kwargs.get('use_best_guess', False)
    min_score = kwargs.get('min_score', 2.0)
    if pmid and not pma:
        pma = pm_fetch.article_by_pmid(pmid)
    try:
        results = crossref.query_from_PubMedArticle(pma)
    except CrossRefConnectionError:
        return None
    return crossref.get_top_result(results, crossref.last_params, use_best_guess, min_score=min_score)

def PubMedArticle2doi(pma, use_best_guess=False, min_score=2.0):
    '''starting with a PubMedArticle object, use CrossRef to find a DOI for given article.

        :param: pma (PubMedArticle object)
        :param: use_best_guess (bool) [default: False]
        :param: min_score (float) [default: 2.0]
    '''
    _start_engines()
    result = _protected_crossref_query(pma=pma, use_best_guess=use_best_guess, min_score=min_score)
    if result:
        return result['doi']
    else:
        return None


def PubMedArticle2doi_with_score(pma, use_best_guess=False, min_score=2.0):
    '''Starting with a PubMedArticle object, use CrossRef to find a DOI for given article.

        Returns a tuple containing the DOI and the score CrossRef returned for the
        lookup.  If there was no good result above the min_score threshold, the tuple 
        will contain (None, 0.0).

        :param: pma (PubMedArticle object)
        :param: use_best_guess (bool) [default: False]
        :param: min_score (float) [default: 2.0]
        :return: (doi, score) (string, float) or (None, 0.0)
    '''

    _start_engines()
    result = _protected_crossref_query(pma=pma, use_best_guess=use_best_guess, min_score=min_score)
    if result:
        return (result['doi'], result['score'])
    else:
        return (None, 0.0)

def pmid2doi(pmid, use_best_guess=False, min_score=2.0):
    '''starting with a pubmed ID, lookup article in pubmed. If DOI found in PubMedArticle object,
        return it.  Otherwise, use CrossRef to find the DOI for given article.

        :param: pmid (string or int)
        :param: use_best_guess (bool) [default: False]
        :param: min_score (float) [default: 2.0]
    '''
    # let MetaPubError pass back to the caller if pmid is not for realz..
    _start_engines()
    pma = pm_fetch.article_by_pmid(pmid)
    if pma.doi:
        return pma.doi
    return PubMedArticle2doi(pma, use_best_guess, min_score=2.0)

def pmid2doi_with_score(pmid, use_best_guess=False, min_score=2.0):
    '''Starting with a pubmed ID, lookup article in pubmed. 
        
        If DOI found in PubMedArticle object, that doi and 10.0 for the doi_score, 
        i.e. the tuple (doi, 10.0).
    
        Otherwise, use CrossRef to find the DOI for given article and return (doi, crossref_doi_score).

        :param: pmid (string or int)
        :param: use_best_guess (bool) [default: False]
        :param: min_score (float) [default: 2.0]
        :return: tuple (doi, doi_score)
    '''
    _start_engines()
    pma = pm_fetch.article_by_pmid(pmid)
    if pma.doi:
        return (pma.doi, 10.0)
    return PubMedArticle2doi_with_score(pma, use_best_guess, min_score=2.0)

def doi2pmid(doi, use_best_guess=False, min_score=2.0):
    '''uses CrossRef and PubMed eutils to lookup a PMID given a known doi.
        Warning: does NO validation (use in combo with metapub.text_mining).

        If a PMID can be found, return it. Otherwise return None.
    '''
    # for PMA, skip the validation; some pubmed XML has weird partial strings for DOI.
    # We should allow people to search using these oddball strings.
    _start_engines()
    doi = doi.strip()
    try:
        pma = pm_fetch.article_by_doi(doi)
        return pma.pmid
    except:
        pass

    try:
        results = crossref.query(doi)
    except CrossRefConnectionError:
        return None

    if results:
        top_result = crossref.get_top_result(results, crossref.last_params, use_best_guess, min_score=min_score)
        pmids = pm_fetch.pmids_for_citation(**top_result['slugs'])
        if len(pmids) == 1:
            if pmids[0] == 'NOT_FOUND':
                return None
            return str(pmids[0])
        elif len(pmids) == 0:
            return None
        else:
            return 'AMBIGUOUS'
    else:
        return None




from __future__ import division
from datetime import datetime, timedelta

def datetime_to_timestamp(dt, epoch=datetime(1970,1,1)):
    '''takes a python datetime object and converts it to a Unix timestamp.

    This is a non-timezone-aware function.

    :param: dt (datetime)
    :return: timestamp (float)
    '''
    td = dt - epoch
    # return td.total_seconds()
    return (td.microseconds + (td.seconds + td.days * 86400) * 10**6) / 10**6 




# TODO
#
#12035837: Can J Neurol Sci -- no URL because No URL format for Journal Can J Neurol Sci
#       ???

#12036192: Clin. Nephrol. -- no URL because No URL format for Journal Clin Nephrol
#       http://www.dustri.com/uploads/repository/21/1239843263_CN83_suppl_01_090.pdf

#12037434: Eur. Neurol. -- no URL because No URL format for Journal Eur Neurol
#12055472: Retina (Philadelphia, Pa.) -- no URL because No URL format for Journal Retina (Philadelphia, Pa)
#12063046: Jpn. J. Ophthalmol. -- no URL because No URL format for Journal Jpn J Ophthalmol
#12063482: J. Am. Acad. Dermatol. -- no URL because No URL format for Journal J Am Acad Dermatol
#12056405: Genes Cells -- no URL because No URL format for Journal Genes Cells
#12065946: Fetal. Diagn. Ther. -- no URL because No URL format for Journal Fetal Diagn Ther
#12066726: Br J Anaesth -- no URL because No URL format for Journal Br J Anaesth
#12071635: J. Neuropathol. Exp. Neurol. -- no URL because No URL format for Journal J Neuropathol Exp Neurol
#12071824: Arch Dermatol -- no URL because No URL format for Journal Arch Dermatol
#12070551: Saudi Med J -- no URL because No URL format for Journal Saudi Med J
#12074273: Circ. J. -- no URL because No URL format for Journal Circ J
#12076704: Oral Oncol. -- no URL because No URL format for Journal Oral Oncol
#12080609: Rinsho Shinkeigaku -- no URL because No URL format for Journal Rinsho Shinkeigaku
#12162770: Arch. Otolaryngol. Head Neck Surg. -- no URL because No URL format for Journal Arch Otolaryngol Head Neck Surg
#12170759: Cancer Biol. Ther. -- no URL because No URL format for Journal Cancer Biol Ther
#12173465: Mol. Biol. (Mosk.) -- no URL because No URL format for Journal Mol Biol (Mosk)
#12173720: Mayo Clin. Proc. -- no URL because No URL format for Journal Mayo Clin Proc
#12202459: Endocr. Rev. -- no URL because No URL format for Journal Endocr Rev
#12087194: Genes Genet. Syst. -- no URL because No URL format for Journal Genes Genet Syst
#12049533: Arch Surg -- no URL because No URL format for Journal Arch Surg
#10458450: J. Urol. -- no URL because No URL format for Journal J Urol
#10458483: Tohoku J. Exp. Med. -- no URL because No URL format for Journal Tohoku J Exp Med
#10458491: Tohoku J. Exp. Med. -- no URL because No URL format for Journal Tohoku J Exp Med
#10470409: Andrologia -- no URL because No URL format for Journal Andrologia
#10474162: Dev. Dyn. -- no URL because No URL format for Journal Dev Dyn
#10520237: Ophthalmic Genet. -- no URL because No URL format for Journal Ophthalmic Genet
#10521250: Ophthalmic Genet. -- no URL because No URL format for Journal Ophthalmic Genet
#21199372: no URL because No URL format for Journal Basic Clin Pharmacol Toxicol
#15533574: no URL because No URL format for Journal Int J Pediatr Otorhinolaryngol
#17100396: no URL because No URL format for Journal J Med Assoc Thai
#17415575: no URL because No URL format for Journal Arch. Dermatol. Res.
#17415800: no URL because No URL format for Journal Mov. Disord.
#17416296: no URL because No URL format for Journal Arch. Med. Res.
#17143317: no URL because No URL format for Journal Nat Clin Pract Endocrinol Metab
#17145028: no URL because No URL format for Journal Med Clin (Barc)
#17145065: no URL because No URL format for Journal Mutat Res
#15452722: no URL because No URL format for Journal Graefes Arch Clin Exp Ophthalmol
#EBioMedicine

# HARDER CASES:
#
#10413889: Medicina (B Aires) -- no URL because No URL format for Journal Medicina (B Aires)
#12027577: Seizure -- no URL because No URL format for Journal Seizure
#10415464: Ophthalmic Genet. -- no URL because No URL format for Journal Ophthalmic Genet
#12015062: Zhonghua Xue Ye Xue Za Zhi -- no URL because No URL format for Journal Zhonghua Xue Ye Xue Za Zhi
#12015065: Zhonghua Xue Ye Xue Za Zhi -- no URL because No URL format for Journal Zhonghua Xue Ye Xue Za Zhi
#12032748: Int. J. Obes. Relat. Metab. Disord. -- no URL because No URL format for Journal Int J Obes Relat Metab Disord


#TODO: journals whose articles can best be accessed by loading up via dx.doi.org
#       and then doing some text replacement on the URL.
doi2step_journals = [ 
                      'J Public Health Policy'  # ex. http://www.palgrave-journals.com/jphp/journal/v36/n2/pdf/jphp201453a.pdf
                    ]


todo_journals = {
    'Pharmacol Rep': { 'example': 'http://www.ncbi.nlm.nih.gov/pubmed/?term=23238479[uid] --> www.if-pan.krakow.pl/pjp/pdf/2012/5_1234.pdf' },
    'Med Sci Monit': { 'example': 'http://www.medscimonit.com/download/index/idArt/869530' },
    'Asian Pac J Cancer Prev': { 'example': 'http://www.apocpcontrol.org/paper_file/issue_abs/Volume12_No7/1771-1776%20c%206.9%20Lei%20Zhong.pdf' },
    'Rev Esp Cardiol': { 'example': 'http://www.revespcardiol.org/en/linkresolver/articulo-resolver/13131646/' },
    'Ann Dermatol Venereol': { 'example': 'http://www.em-consulte.com/article/152959/alertePM' },
    'Mol Cells': { 'example': 'http://www.molcells.org/journal/view.html?year=2004&volume=18&number=2&spage=141 --> http://pdf.medrang.co.kr/KSMCB/2004/018/mac-18-2-141.pdf'},
    'Mol Vis': { 'example': 'http://www.molvis.org/molvis/v10/a45/ --> http://www.molvis.org/bin/pdf.cgi?Zheng,10,45'},
    'Singapore Med J': { 'example': 'http://www.sma.org.sg/smj/4708/4708cr4.pdf' },
    'Rev Port Cardiol': { 'example': '16335287: http://www.spc.pt/DL/RPC/artigos/74.pdf' },
    'World J Gastroenterol': { 'example': 'http://www.wjgnet.com/1007-9327/full/v11/i48/7690.htm --> http://www.wjgnet.com/1007-9327/pdf/v11/i48/7690.pdf' },
    'Genet Mol Res': { 'example': '24668667: http://www.geneticsmr.com/articles/2992 --> http://www.geneticsmr.com//year2014/vol13-1/pdf/gmr2764.pdf' },
    'Arq Bras Cardiol': { 'example': '20944894: http://www.scielo.br/pdf/abc/v95n5/en_aop13210.pdf' },
    'Arq Bras Endocrinol Metabol': { 'example': '15611820: http://www.scielo.br/pdf/abem/v48n1/19521.pdf' },
    'Neoplasma': { 'example': '17319787: http://www.elis.sk/download_file.php?product_id=1006&session_id=skl2f3grcd19ebnie17u15a571' },
    'Clinics (Sao Paulo)': { 'example': '17823699: http://www.scielo.br/scielo.php?script=sci_arttext&pid=S1807-59322007000400003' },
    'Asian J Androl': {'example': '18097502: http://www.asiaandro.com/Abstract.asp?doi=10.1111/j.1745-7262.2008.00376.x' },
    'Anesthesiology': {'example': '18212565: http://dx.doi.org/10.1097/01.anes.0000299431.81267.3e --> html w/ <a id="pdfLink" data-article-url="THE_URL">' },
    'Nat Prod Commun': {'example': '19634325 (no direct link found yet) -- http://www.naturalproduct.us/' },
    'Oncotarget': {'example': '26008984 (pii = 3900) --> http://www.impactjournals.com/oncotarget/index.php?journal=oncotarget&page=article&op=view&path=3900' },
    'Clin Ter': {'example': '25756258 --> dx.doi.org/10.7417/CT.2015.1799 --> parse page to get PDF' },
    'J Pediatr (Rio J)': { 'example': '17102902 --> dx.doi.org/10.2223/JPED.1550 --> http://www.jped.com.br/conteudo/06-82-06-437/port.pdf' },
    'Teach Learn Med': { 'example': '17144842 --> dx.doi.org/10.1207/s15328015tlm1804_13 --> pdf link?' },
    }


format_templates = {
    'acs': 'http://pubs.acs.org/doi/pdf/{a.doi}',
    'akademii': 'http://www.akademiai.com/content/{a.pii}/fulltext.pdf',
    'ats': 'http://www.atsjournals.org/doi/pdf/{a.doi}',
    'informa': 'http://informahealthcare.com/doi/pdf/{a.doi}',
    'lancet': 'http://www.thelancet.com/pdfs/journals/{ja}/PII{a.pii}.pdf',
    'liebert': 'http://online.liebertpub.com/doi/pdf/{a.doi}',
    'plos': 'http://www.plosbiology.org/article/fetchObjectAttachment.action?url=info:doi/{a.doi}&representation=PDF',
    'wiley': 'http://onlinelibrary.wiley.com/doi/{a.doi}/pdf',
    'jci': 'http://www.jci.org/articles/view/{a.pii}/pdf',
    }

# JCI == Journal of Clinical Investigation
jci_journals = [ 'J Clin Invest' ]

# TODO: Dustri Dance
# Dustri: see http://www.dustri.com/journals-in-english.html
dustri_journals = ( 'Clin Nephrol', 'Int J Clin Pharmacol Ther', 'Clin Neuropathol' )

# Portlan Press Biochemical Society journals: mostly VIP.
#TODO: detect and redo urls for "early" releases, e.g.:
#      http://www.clinsci.org/content/ppclinsci/early/2015/06/10/CS20150073.full.pdf
biochemsoc_journals = { 'Biochem J': { 'host': 'www.biochemj.org', 'ja': 'ppbiochemj' },
                        'Clin Sci': { 'host': 'www.clinsci.org', 'ja': 'ppclinsci' },
                      }
biochemsoc_format = 'http://{host}/content/{ja}/{a.volume}/{a.issue}/{a.first_page}.full.pdf'

lancet_journals = {
    'Lancet': { 'ja': 'lancet' },
    'Lancet Diabetes Endocrinol': { 'ja': 'landia' },
    'Lancet Glob Health': { 'ja': 'langlo' },
    'Lancet Haematol': { 'ja': 'lanhae' },
    'Lancet HIV': { 'ja': 'lanhiv' },
    'Lancet Infect Dis': { 'ja': 'laninf' },
    'Lancet Neurol': { 'ja': 'laneur' },
    'Lancet Oncol': { 'ja': 'lanonc' },
    'Lancet Psychiatry': { 'ja': 'lanpsy' },
    'Lancet Respir Med': { 'ja': 'lanres' },
    }


# the SD journals are also represented in simple_formats_pii
# "piit" means pii-translated (i.e. punctuation removed)
sciencedirect_url = 'http://www.sciencedirect.com/science/article/pii/{piit}'
sciencedirect_journals = (
    'Ann Genet',
    'Am J Pathol',
    'Am J Prev Med',
    'Ambul Pediatr',
    'Appetite',
    'Arch Biochem Biophys',
    'Arch Pediatr',
    'Atherosclerosis',
    'Blood Cells Mol Dis',
    'Biochem Biophys Res Commun' ,
    'Biochem Pharmacol',
    'Biochim Biophys Acta',
    'Br J Oral Maxillofac Surg',
    'Child Abuse Negl',
    'Clin Chim Acta',
    'Clin Immunol',
    'Contemp Clin Trials',
    'Crit Care Clin',
    'Fam Pract',
    'FEBS Lett',
    'Eur J Cancer',
    'Eur J Med Genet',
    'Exp Parasitol',
    'Gene',
    'Genomics',
    'Gynecol Obstet Fertil',
    'Hepatol Res',
    'Hum Immunol',
    'Infect Genet Evol',
    'J Acad Nutr Diet',
    'J Am Coll Cardiol',
    'J Am Diet Assoc',
    'J Adolesc Health',
    'J Autoimmun',
    'J Environ Sci (China)',
    'J Health Econ',
    'J Mol Biol',
    'J Neurol Sci',
    'J Pediatr',
    'J Pediatr Health Care',
    'J Reprod Immunol',
    'J Steroid Biochem Mol Biol',
    'J Struct Biol',
    'J Urol',
    'Leuk Res',
    'Life Sci',
    'Mol Cell Endocrinol',
    'Mol Cell Probes',
    'Mol Genet Metab',
    'Mol Immunol',
    'Mutat Res',
    'Neuromuscul Disord',
    'Neurosci Lett',
    'Neurosci Res',
    'Neuroscience',
    'Neurotoxicology',
    'Nitric Oxide',
    'Nutr Metab Cardiovasc Dis',
    'Patient Educ Couns',
    'Pediatr Nurs',
    'Pediatr Pulmonol',
    'Sci Total Environ',
    'Soc Sci Med',
    'Toxicol In Vitro',
    'Trends Biochem Sci',
    'Virology',
    'Virus Res',
    )


jama_journals = (
     'Arch Gen Psychiatry',
     'Arch Neurol',
     'Arch Ophthalmol',
     'JAMA',
     'JAMA Dermatol',
     'JAMA Facial Plast Surg',
     'JAMA Intern Med',
     'JAMA Neurol',
     'JAMA Oncol',
     'JAMA Ophthalmol',
     'JAMA Otolaryngol Head Neck Surg',
     'JAMA Pediatr',
     'JAMA Psychiatry',
     'JAMA Surg',
     )

# TODO
# doiserbia (Library of Serbia) articles can be grabbed by doing the_doi_2step,
# then ...?
doiserbia_journals = ['Genetika']

# JSTAGE: mostly free (yay)
# Examples:
# J Biochem: https://www.jstage.jst.go.jp/article/biochemistry1922/125/4/125_4_803/_pdf
# Drug Metab Pharmacokinet: https://www.jstage.jst.go.jp/article/dmpk/20/2/20_2_144/_article --> https://www.jstage.jst.go.jp/article/dmpk/20/2/20_2_144/_pdf
jstage_journals = [
    'Intern Med',
    'J Periodontol',
    'J Biochem',
    'Drug Metab Pharmacokinet',
    'Endocr J',
    ]

# cell journals
#cell_format = 'http://download.cell.com{ja}/pdf/PII{pii}.pdf'
cell_format = 'http://www.cell.com{ja}/pdf/{pii}.pdf'
cell_journals = {
    'Am J Hum Genet': { 'ja': '/AJHG' },
    'Biophys J': { 'ja': '/biophysj' },
    'Cancer Cell': { 'ja': '/cancer-cell' },
    'Cell': { 'ja': '' },
    'Cell Host Microbe': {'ja': '/cell-host-microbe' },
    'Cell Metab': {'ja': '/cell-metabolism' },
    'Cell Stem Cell': {'ja': '/cell-stem-cell' },
    'Chem Biol': {'ja': '/chemistry-biology' },
    'Curr Biol': {'ja': '/current-biology' },
    'Dev Cell': {'ja': '/developmental-cell' },
    'Immunity': {'ja': '/immunity' },
    'Mol Cell': {'ja': '/molecular-cell' },
    'Neuron': {'ja': '/neuron' },
    'Structure': {'ja': '/structure' },
    'Trends Mol Med': { 'ja': '/trends' },
    }

# nature journals -- COMPLETE
nature_format = 'http://www.nature.com/{ja}/journal/v{a.volume}/n{a.issue}/pdf/{a.pii}.pdf'
nature_journals = {
    'Eur J Clin Nutr': { 'ja': 'ejcn' },
    'Eur J Hum Genet': { 'ja': 'ejhg' },
    'Eye (Lond)': { 'ja': 'eye' },
    'Genes Immun': { 'ja': 'gene' },
    'Genet Med': { 'ja': 'gim' },
    'J Invest Dermatol': { 'ja': 'jid' },
    'J Hum Genet': { 'ja': 'jhg' },
    'Kidney Int': { 'ja': 'ki' },
    'Leukemia': { 'ja': 'leu' },
    'Mod Pathol': { 'ja': 'modpathol' },
    'Mol Psychiatry': { 'ja': 'mp' },
    'Nature': { 'ja': 'nature' },
    'Nat Chem': { 'ja': 'nchem' },
    'Nat Clin Pract Endocrinol Metab': { 'ja': 'nrendo' },
    'Nat Clin Pract Cardiovasc Med': { 'ja': 'nrcardio' },
    'Nat Clin Pract Oncol': { 'ja': 'nrclinonc' },
    'Nat Clin Pract Gastroenterol Hepatol': { 'ja': 'nrgastro' },
    'Nat Clin Pract Urol': { 'ja': 'nrurol' },
    'Nat Clin Pract Neurol': { 'ja': 'nrneurol' },
    'Nat Clin Pract Nephrol': { 'ja': 'nrneph' },
    'Nat Clin Pract Rheumatol': { 'ja': 'nrrheum' },
    'Nat Genet': { 'ja': 'ng' },
    'Nat Commun': {'ja': 'ncomms' },
    'Nat Nanotechnol': { 'ja': 'nnano' },
    'Nat Neurosci': { 'ja': 'neuro' },
    'Nat Mater': { 'ja': 'nmat' },
    'Nat Med': { 'ja': 'nm' },
    'Nat Methods': { 'ja': 'nmeth' },
    'Nat Protoc': { 'ja': 'nprot' },
    'Nat Rev Drug Discov': { 'ja': 'nrd' },
    'Nat Rev Cardiol': { 'ja': 'nrcardio' },
    'Nat Rev Clin Oncol': { 'ja': 'nrclinonc' },
    'Nat Rev Endocrinol': { 'ja': 'nrendo' },
    'Nat Rev Genet': { 'ja': 'nrg' },
    'Nat Rev Gastroenterol Hepatol': { 'ja': 'nrgastro' },
    'Nat Rev Nephrol': { 'ja': 'nrneph' },
    'Nat Rev Neurol': { 'ja': 'nrneurol' },
    'Nat Rev Rheumatol': { 'ja': 'nrrheum' },
    'Nat Rev Urol': { 'ja': 'nrurol' },
    'Nat Rev Immunol': { 'ja': 'nri' },
    'Neuropsychopharmacology': { 'ja': 'npp' },
    'Oncogene': { 'ja': 'onc' },
    'Pediatr Res': { 'ja': 'pr' },
    }


# simple_formats_pmid: links to PDFs that can be constructed using the pubmed ID
simple_formats_pmid = {
    'Medicina (B Aires)': 'http://www.medicinabuenosaires.com/PMID/{pmid}.pdf',
    }


# simple formats are used for URLs that can be deduced from PubMedArticle XML
#       
#       !ACHTUNG!  informa has been known to block IPs for the capital offense of 
#                  having "More than 25 sessions created in 5 minutes"
#
simple_formats_doi = {
    'Acta Oncol': format_templates['informa'],
    'Ann Hum Biol': format_templates['informa'],
    'Hemoglobin': format_templates['informa'],
    'J Matern Fetal Neonatal Med': format_templates['informa'],
    'Platelets': format_templates['informa'],
    'Ren Fail': format_templates['informa'],
    'Xenobiotica': format_templates['informa'],

    'Am J Public Health': 'http://ajph.aphapublications.org/doi/pdf/{a.doi}',

    'Am J Respir Cell Mol Biol': format_templates['ats'],
    'Am J Respir Crit Care Med': format_templates['ats'],

    'Anal Chem': format_templates['acs'],
    'Biochemistry': format_templates['acs'],
    'J Am Chem Soc': format_templates['acs'],

    'Child Obes': format_templates['liebert'],
    'Genet Test': format_templates['liebert'],
    'Genet Test Mol Biomarkers': format_templates['liebert'],
    'Thyroid': format_templates['liebert'],

    'Endocrinology': 'http://press.endocrine.org/doi/pdf/{a.doi}',
    'Mol Endocrinol': 'http://press.endocrine.org/doi/pdf/{a.doi}',
    'J Periodontol': 'http://www.joponline.org/doi/pdf/{a.doi}',

    'PLoS Biol': format_templates['plos'],
    'PLoS Comput Biol': format_templates['plos'],
    'PLoS Genet': format_templates['plos'],
    'PLoS Med': format_templates['plos'],
    'PLoS ONE': format_templates['plos'],
    'PLoS Pathog': format_templates['plos'],
    'N Engl J Med':  'http://www.nejm.org/doi/pdf/{a.doi}',
    }



simple_formats_pii = {
    'Am Heart J': 'http://www.ahjonline.com/article/{a.pii}/pdf', #ScienceDirect
    'Am J Cardiol': 'http://www.ajconline.org/article/{a.pii}/pdf', #ScienceDirect
    'Am J Ophthalmol': 'http://www.ajo.com/article/{a.pii}/pdf', #ScienceDirect
    'Am J Med': 'http://www.amjmed.com/article/{a.pii}/pdf', #ScienceDirect
    'Atherosclerosis': 'http://www.atherosclerosis-journal.com/article/{a.pii}/pdf', #ScienceDirect
    'Arch Med Res': 'http://www.arcmedres.com/article/{a.pii}/pdf', #ScienceDirect
    'Biol Psychiatry': 'http://www.biologicalpsychiatryjournal.com/article/{a.pii}/pdf', #ScienceDirect
    'Bone': 'http://www.thebonejournal.com/article/{a.pii}/pdf', #ScienceDirect
    'Brain Dev': 'http://www.brainanddevelopment.com/article/{a.pii}/pdf', #ScienceDirect
    'Cancer Cell Int': 'http://www.cancerci.com/content/pdf/{a.pii}.pdf', #Biomed Central
    'Cancer Genet Cytogenet': 'http://www.cancergeneticsjournal.org/article/{a.pii}/pdf', #ScienceDirect
    'Cancer Lett': 'http://www.cancerletters.info/article/{a.pii}/pdf', #ScienceDirect
    'Clin Neurol Neurosurg': 'http://www.clineu-journal.com/article/{a.pii}/pdf', #ScienceDirect
    'Diabetes Res Clin Pract': 'http://www.diabetesresearchclinicalpractice.com/article/{a.pii}/pdf', #ScienceDirect
    'Epilepsy Res': 'http://www.epires-journal.com/article/{a.pii}/pdf', #ScienceDirect
    'Eur J Paediatr Neurol': 'http://www.ejpn-journal.com/article/{a.pii}/pdf', #ScienceDirect
    'Exp Hematol': 'http://www.exphem.org/article/{a.pii}/pdf', #ScienceDirect
    'Fertil Steril': 'http://www.fertstert.org/article/{a.pii}/pdf',    #ScienceDirect
    'Gastroenterology': 'http://www.gastrojournal.org/article/{a.pii}/pdf',
    'Gynecol Oncol': 'http://www.gynecologiconcology-online.net/article/{a.pii}/pdf', #ScienceDirect
    'Heart Rhythm': 'http://www.heartrhythmjournal.com/article/{a.pii}/pdf',
    'Int J Pediatr Otorhinolaryngol': 'http://www.ijporlonline.com/article/{a.pii}/pdf', #ScienceDirect
    'Int J Cardiol': 'http://www.internationaljournalofcardiology.com/article/{a.pii}/pdf', #ScienceDirect
    'J Dermatol': 'http://www.jdsjournal.com/article/{a.pii}/pdf', #ScienceDirect
    'J Mol Cell Cardiol': 'http://www.jmmc-online.com/article/{a.pii}/pdf', #ScienceDirect
    'J Mol Diagn': 'http://jmd.amjpathol.org/article/{a.pii}/pdf',  #ScienceDirect
    'J Neurol Sci': 'http://www.jns-journal.com/article/{a.pii}/pdf',
    'J Pediatr': 'http://www.jpeds.com/article/{a.pii}/pdf',  #ScienceDirect
    'J Pediatr Urol': 'http://www.jpurol.com/article/{a.pii}/pdf',  #ScienceDirect
    'Ophthalmology': 'http://www.aaojournal.org/article/{a.pii}/pdf', #ScienceDirect
    'Orv Hetil': format_templates['akademii'],
    'Med Hypotheses': 'http://www.medical-hypotheses.com/article/{a.pii}/pdf', #ScienceDirect
    'Metabolism': 'http://www.metabolismjournal.com/article/{a.pii}/pdf', #ScienceDirect
    'Metab Clin Exp': 'http://www.metabolismjournal.com/article/{a.pii}/pdf', #ScienceDirect
    'Neurobiol Aging': 'http://www.neurobiologyofaging.org/article/{a.pii}/pdf', #ScienceDirect
    'Neuromuscul Disord': 'http://www.nmd-journal.com/article/{a.pii}/pdf', #ScienceDirect
    'Parkinsonism Relat Disord': 'http://www.prd-journal.com/article/{a.pii}/pdf', #ScienceDirect
    'Pediatr Neurol': 'http://www.pedneur.com/article/{a.pii}/pdf', #ScienceDirect
    'Placenta': 'http://www.placentajournal.org/article/{a.pii}/pdf', #ScienceDirect
    'Surg Neurol': 'http://www.worldneurosurgery.org/article/{a.pii}/pdf', #ScienceDirect
    'Thromb Res': 'http://www.thrombosisresearch.com/article/{a.pii}/pdf', #ScienceDirect
    }

#    The following were taken out of simple_formats_pii but kept in the sciencedirect_journals list:
#    'Mol Genet Metab': 'http://www.mgmjournal.com/article/{a.pii}/pdf', #ScienceDirect


# Many BMC journals start with "BMC" (they're covered automatically) --
#   this list covers the ones that don't.  #TODO: Gather up the ones not in here yet.
BMC_journals = (
    'Diagn Pathol',
    'Genome Biol',
    'Genome Med',
    'Hum Genomics',
    'J Clin Bioinforma',
    'J Transl Med',
    )

# the "aid" is the second half of the DOI string (after the slash)
BMC_format = 'http://www.biomedcentral.com/content/pdf/{aid}.pdf'

# vip = Volume-Issue-Page format 
#       URLs that have the same format except for the host name

vip_format = 'http://{host}/content/{a.volume}/{a.issue}/{a.first_page}.full.pdf'

vip_journals = {
        'Ann Clin Biochem': { 'host': 'acb.sagepub.com' },
        'Am J Clin Pathol': { 'host': 'ajcp.ascpjournals.org' },
        'Am J Hypertens': { 'host': 'ajh.oxfordjournals.org' },
        'Ann Oncol': { 'host' : 'annonc.oxfordjournals.org' },
        'Antimicrob Agents Chemother': { 'host': 'aac.asm.org' }, #TODO: backup_url: pmid lookup strategy, e.g. http://aac.asm.org/cgi/pmidlookup?view=long&pmid=7689822
        'Arterioscler Thromb Vasc Biol': { 'host' : 'atvb.ahajournals.org' },
        'Brain': { 'host': 'brain.oxfordjournals.org' },
        'Breast Cancer Res': { 'host': 'breast-cancer-research.com' },
        'Cancer Discov': { 'host': 'cancerdiscovery.aacrjournals.org' },
        'Cancer Epidemiol Biomarkers Prev': { 'host': 'cebp.aacrjournals.org' },
        'Cancer Res': { 'host': 'cancerres.aacrjournals.org' },
        'Can Fam Physician': { 'host': 'www.cfp.ca' }, #TODO: backup_url: pmid lookup strategy, e.g. http://www.cfp.ca/cgi/pmidlookup?view=long&pmid=19282532
        'Carcinogenesis': { 'host': 'carcin.oxfordjournals.org' } ,
        'Cardiovasc Res': { 'host' : 'cardiovascres.oxfordjournals.org' },
        'Circulation': { 'host': 'circ.ahajournals.org' },
        'Circ Arrhythm Electrophysiol': { 'host': 'circep.ahajournals.org' },
        'Circ Cardiovasc Genet': { 'host' : 'circgenetics.ahajournals.org' },
        'Circ Res': { 'host' : 'circres.ahajournals.org' },
        'Clin Cancer Res': { 'host' : 'clincancerres.aacrjournals.org' },
        'Clin Chem': { 'host' : 'clinchem.org' },
        'Clin Infect Dis': { 'host': 'cid.oxfordjournals.org' },
        'Clin Pediatr': { 'host': 'cpj.sagepub.com' },
        'Clin Pediatr (Phila)': { 'host': 'cpj.sagepub.com' },
        'Diabetes': { 'host': 'diabetes.diabetesjournals.org' },
        'Diabetes Care': { 'host': 'care.diabetesjournals.org' },
        'Drug Metab Dispos': { 'host': 'dmd.aspetjournals.org' },
        'EMBO J': { 'host': 'emboj.embopress.org' }, ##TODO: backup_url: pmid lookup strategy, http://emboj.embopress.org/cgi/pmidlookup?view=long&pmid=9501081
        'Endocr Relat Cancer': { 'host': 'erc.endocrinology-journals.org' },
        'Eur Heart J' : { 'host' : 'eurheartj.oxfordjournals.org' },
        'Eur J Endocrinol' : { 'host' : 'eje-online.org' },
        'FASEB J': { 'host' : 'fasebj.org' },
        'Genome Biol': { 'host' : 'genomebiology.com' },
        'Genome Res': { 'host': 'genome.cshlp.org' },
        'Genes Dev': { 'host': 'genesdev.cshlp.org' },
        'Gut': { 'host' : 'gut.bmj.com' },
        'Haematologica': { 'host' : 'haematologica.org' },
        'Hum Mol Genet': { 'host': 'hmg.oxfordjournals.org' },
        'Hum Reprod': { 'host': 'humrep.oxfordjournals.org' },
        'Hypertension': { 'host': 'hyper.ahajournals.org' },
        'Invest Ophthalmol Vis Sci': { 'host': 'www.iovs.org' },
        'IOVS': { 'host' : 'iovs.org' },
        'J Aging Health': { 'host': 'jah.sagepub.com' }, ##TODO: backup_url: pmid lookup strategy, http://jah.sagepub.com/cgi/pmidlookup?view=long&pmid=20056814
        'J Am Soc Nephrol': { 'host' : 'jasn.asnjournals.org' },
        'J Bacteriol': { 'host': 'jb.asm.org' },  #TODO: backup_url: pmid lookup strategy, http://jb.asm.org/cgi/pmidlookup?view=long&pmid=7683021
        'J Biol Chem': { 'host': 'www.jbc.org' },   #TODO backup_url: pmid lookup strategy, e.g. http://www.jbc.org/cgi/pmidlookup?view=long&pmid=14722075
        'J Cell Biol': { 'host' : 'jcb.rupress.org' },
        'J Cell Sci': { 'host' : 'jcs.biologists.org' },
        'J Child Neurol': { 'host': 'jcn.sagepub.com' },
        'J Clin Endocrinol Metab': { 'host': 'jcem.endojournals.org' },
        'J Clin Oncol': { 'host': 'jco.ascopubs.org' },
        'J Dent Res': { 'host': 'jdr.sagepub.com' },
        'J Gerontol A Biol Sci Med Sci': { 'host': 'biomedgerontology.oxfordjournals.org' },
        'J Hum Lact': { 'host': 'jhl.sagepub.com' },
        'J Immunol': { 'host' : 'jimmunol.org' },
        'J Infect Dis': { 'host': 'jid.oxfordjournals.org' },
        'J Lipid Res': { 'host': 'www.jlr.org' },
        'J Med Genet': { 'host': 'jmg.bmj.com' },
        'J Mol Endocrinol': { 'host': 'jme.endocrinology-journals.org' },
        'J Natl Cancer Inst': { 'host': 'jnci.oxfordjournals.org' },
        'J Neurol Neurosurg Psychiatry': { 'host': 'jnnp.bmj.com' },
        'J Neurosci': { 'host' : 'jneurosci.org' },
        'J Nutr': { 'host': 'jn.nutrition.org' }, #TODO:  backup_url: pmid lookup strategy, http://jn.nutrition.org/cgi/pmidlookup?view=long&pmid=10736367
        'J Pharmacol Exp Ther': { 'host': 'jpet.aspetjournals.org' },
        'J Rheumatol': { 'host': 'www.jrheum.org' },
        'J Virol': { 'host': 'jvi.asm.org' },
        'Mol Biol Cell': { 'host' : 'molbiolcell.org' },
        'Mol Cell Biol': { 'host': 'mcb.asm.org' },
        'Mol Canc Therapeut': { 'host' : 'mct.aacrjournals.org' },
        'Mol Hum Reprod': { 'host': 'molehr.oxfordjournals.org' },
        'Mol Pharmacol': { 'host' : 'molpharm.aspetjournals.org' },
        'Neurology': { 'host' : 'neurology.org' },
        'Nephrol Dial Transplant': { 'host': 'ndt.oxfordjournals.org' },
        'Nucleic Acids Res': { 'host' : 'nar.oxfordjournals.org' },
        'Orphanet J Rare Dis': { 'host' : 'ojrd.com' },
        'Pediatrics': { 'host': 'pediatrics.aappublications.org'},
        'Plant Cell': { 'host': 'www.plantcell.org' }, #TODO:  backup_url: pmid lookup strategy, e.g. http://www.plantcell.org/cgi/pmidlookup?view=long&pmid=9501112
        'Plant Cell Physiol': { 'host': 'pcp.oxfordjournals.org' },
        'Proc Natl Acad Sci USA': { 'host': 'pnas.org' },
        'Protein Eng': { 'host': 'peds.oxfordjournals.org' },
        'QJM': { 'host': 'qjmed.oxfordjournals.org'},
        'Science': { 'host': 'sciencemag.org' },
        'Thorax': { 'host': 'thorax.bmj.com' },
        }

# volume-issue-page type URLs but with a nonstandard baseurl construction.
# e.g. Blood: http://www.bloodjournal.org/content/bloodjournal/79/10/2507.full.pdf
#      BMJ:   http://www.bmj.com/content/bmj/350/bmj.h3317.full.pdf
#      Thorax: http://thorax.bmj.com/content/early/2015/06/23/thoraxjnl-2015-207199.full.pdf+html

# no trailing slash in baseurl (please)
vip_journals_nonstandard = { 
	#TODO: backup_url: pmid lookup strategy, e.g. http://www.bloodjournal.org/cgi/pmidlookup?view=long&pmid=1586703
    'Blood': 'http://www.bloodjournal.org/content/bloodjournal/{a.volume}/{a.issue}/{a.first_page}.full.pdf',
    'BMJ':   'http://www.bmj.com/content/bmj/{a.volume}/bmj.{a.first_page}.full.pdf', 
    }

# Science (AAAS) -- requires login "as a courtesy to the community".  Mkay.
aaas_format = 'http://{ja}.sciencemag.org/content/{a.volume}/{a.issue}/{a.pages}.full.pdf'
aaas_journals = {
        'Science': { 'ja': 'www' },
        'Sci Adv': { 'ja': 'advances' },
        'Sci Signal': { 'ja': 'stke' },
        'Sci Transl Med': { 'ja': 'stm' },
    }


# Spandidos: volume/issue/firstpage AND a journal abbreviation. Fancy.
spandidos_format = 'http://www.spandidos-publications.com/{ja}/{a.volume}/{a.issue}/{a.first_page}/download'
spandidos_journals = {
        'Int J Oncol': { 'ja' : 'ijo' },
        'Int J Mol Med': { 'ja': 'ijmm' },
        'Oncol Lett': { 'ja' : 'ol' },
        'Oncol Rep': { 'ja' : 'or' },
    }


wiley_journals = (
    'Acta Neurol Scand',
    'Acta Ophthalmol Scand',
    'Ann Hum Genet',
    'Ann Neurol',
    'Am J Hematol',
    'Am J Med Genet',
    'Am J Med Genet A',
    'Am J Med Genet B Neuropsychiatr Genet',
    'Arthritis Rheum',
    'Australas J Dermatol',
    'Basic Clin Pharmacol Toxicol',
    'BJU Int',
    'Breast J',
    'Br J Dermatol',
    'Br J Haematol',
    'Cancer',
    'Cancer Sci',
    'Clin Endocrinol (Oxf)',
    'Clin Exp Dermatol',
    'Clin Genet',
    'Clin Pharmacol Ther',
    'Diabet Med',
    'Dev Med Child Neurol',
    'Electrophoresis',
    'Environ Mol Mutagen',
    'Eur J Biochem',
    'Eur J Clin Invest',
    'Eur J Haematol',
    'Eur J Neurol',
    'Exp Dermatol',
    'Genes Chromosomes Cancer',
    'Genet Epidemiol',
    'Haemophilia',
    'Head Neck',
    'Headache',
    'Hepatology',
    'Hum Brain Mapp',
    'Hum Mutat',
    'Immunol Rev',
    'Int J Cancer',
    'Int J Immunogenet',
    'Int J Lab Hematol',
    'J Am Assoc Nurse Pract',
    'J Bone Miner Res',
    'J Dermatol',
    'J Eur Acad Dermatol Venereol',
    'J Gastroenterol Hepatol',
    'J Intern Med',
    'J Med Virol',
    'J Neurochem',
    'J Orthop Res',
    'J Pathol',
    'J Rural Health',
    'J Sch Health',
    'J Thromb Haemost',
    'J Viral Hepat',
    'Med Educ',
    'Mol Carcinog',
    'Mol Microbiol',
    'Mol Plant Pathol',
    'Mov Disord',
    'Muscle Nerve',
    'Neuropathol Appl Neurobiol',
    'Nihon Shokakibyo Gakkai Zasshi',
    'Obesity (Silver Spring)',
    'Pain Med',
    'Pediatr Blood Cancer',
    'Pediatr Diabetes',
    'Pediatr Int',
    'Prenat Diagn',
    'Prostate',
    'Protein Sci',
    'Proteins',
    'Scand J Immunol',
    'Tissue Antigens',
    'Transfus Med',
    'Transfusion',
    'Vox Sang',
    )

#TODO: De Gruyter (publisher)
#
# examples:
# 26110471 Arh Hig Rada Toksikol http://www.degruyter.com/view/j/aiht.2015.66.issue-2/aiht-2015-66-2582/aiht-2015-66-2582.xml
#12199334: J. Pediatr. Endocrinol. Metab. -- no URL because No URL format for Journal J Pediatr Endocrinol Metab
#12199344: J. Pediatr. Endocrinol. Metab. -- no URL because No URL format for Journal J Pediatr Endocrinol Metab
#
# load by dx.doi.org: http://dx.doi.org/10.2478/cdth-2014-0001
#       --> http://www.degruyter.com/view/j/cdth.2014.1.issue-1/cdth-2014-0001/cdth-2014-0001.xml
#   PDF --> http://www.degruyter.com/dg/viewarticle.fullcontentlink:pdfeventlink/$002fj$002fcdth.2014.1.issue-1$002fcdth-2014-0001$002fcdth-2014-0001.pdf?t:ac=j$002fcdth.2014.1.issue-1$002fcdth-2014-0001$002fcdth-2014-0001.xml
#
#

#TODO: Taylor & Francis (publisher)
#
# e.g. 25121990: http://www.tandfonline.com/doi/pdf/10.1080/10410236.2014.943634
taylorfrancis_journals = ( 
        'Health Commun',
        )

# Below: Journals with really annoying paywalls guarding their precious secrets.
schattauer_journals = [
    'Thromb Haemost',
    ]

# Royal Society of Chemistry
RSC_journals = [ 'Nat Prod Rep' ]

wolterskluwer_journals = [
    'AIDS',
    'Blood Coagul Fibrinolysis',
    'Clin Dysmorphol',
    'Curr Opin Hematol',
    'Eur J Gastroenterol Hepatol',
    'Fam Community Health',
    'J Dev Behav Pediatr',
    'J Glaucoma',
    'J Hypertens',
    'J Investig Med',
    'J Pediatr Hematol Oncol',
    'J Pediatr Gastroenterol Nutr',
    'J Trauma',
    'Medicine (Baltimore)',
    'Neuroreport',
    'Obstet Gynecol',
    'Pediatr Emerg Care',
    'Pediatr Infect Dis J',
    'Pharmacogenet Genomics',
    'Pharmacogenetics',
    'Plast Reconstr Surg',
    'Psychiatr Genet',
    ]

# karger: mostly paywalled, but sometimes...
# http://www.karger.com/Article/Pdf/351538

karger_journals = [
    'Acta Haematol',
    'Ann Nutr Metab',
    'Cell Physiol Biochem',
    'Cytogenet Genome Res',
    'Dermatology (Basel)',
    'Horm Res',
    'Hum Hered',
    'Nephron',
    'Nephron Physiol',
    ]

# springer is mostly paywalled, but sometimes...
# http://link.springer.com/content/pdf/10.1007%2Fs13238-015-0153-5.pdf
springer_journals = [
    'Acta Neuropathol',
    'Ann Hematol',
    'Ann Surg Oncol',
    'Arch Dermatol Res',
    'Biochem Genet',
    'Breast Cancer Res Treat',
    'Calcif Tissue Int',
    'Cell Mol Neurobiol',
    'Diabetologia',
    'Eur J Pediatr',
    'Eur J Nutr',
    'Fam Cancer',
    'Graefes Arch Clin Exp Ophthalmol',
    'HNO',
    'Hum Genet',
    'Immunogenetics',
    'Int J Colorectal Dis',
    'J Bone Miner Metab',
    'J Clin Immunol',
    'J Endocrinol Invest',
    'J Inherit Metab Dis',
    'J Neural Transm',
    'J Neurol',
    'J Mol Evol',
    'J Mol Med',
    'J Mol Med (Berl)',
    'J Mol Neurosci',
    'Matern Child Health J',
    'Mod Rheumatol',
    'Neurogenetics',
    'Ophthalmologe',
    'Osteoporos Int',
    'Pediatr Nephrol',
    'Pharm Res',
    'Physiol Genomics',
    'Protein Cell',
    'Rheumatol Int',
    'Tumour Biol',
    'Virchows Arch',
    'World J Surg',
    ]

# thieme journals so far don't seem to have any open access content.
# example links to article page: https://www.thieme-connect.com/DOI/DOI?10.1055/s-0028-1085467
#           https://www.thieme-connect.com/DOI/DOI?10.1055/s-2007-1004566
thieme_journals = ['Neuropediatrics', 'Semin Vasc Med', 'Exp Clin Endocrinol Diabetes',
                    'Int J Sports Med']

weird_paywall_publishers = ['J Ment Health Policy Econ' ]

paywall_journals = schattauer_journals + wolterskluwer_journals + thieme_journals + weird_paywall_publishers


#All in PMC (no need to write formats for):
 #Nat Comput
 #Nat Clim Chang
 #Nat Geosci
 #Nat Resour Model
 #Nat Lang Linguist Theory
 #Nat Photonics
 #Nat Phys
 #Nat Prod Bioprospect
 #Nat Lang Eng
 #Nat Rep Stem Cells
 #Nat Sci (Irvine)
 #Nat Sci Sleep



import os, unittest

from metapub import CrossRef


TEST_CACHEDIR = 'tests/testcachedir'

class TestCrossRef(unittest.TestCase):

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_configurable_cachedir(self):
        '''test that `cachedir` keyword argument is fully supported in modes:

        cachedir == 'default'   <-- assumed working since other tests use this.
        cachedir is None
        cachedir is 'some/path'
        cachedir is '~/path'
        '''

        cachedir = TEST_CACHEDIR
        # start with cachedir==None; test that no cachedir is created.
        cr = CrossRef(cachedir=None)
        assert not os.path.exists(cachedir)
        assert cr._cache_path is None

        cr = CrossRef(cachedir=cachedir)
        assert os.path.exists(cachedir)
        assert cr._cache_path is not None

        os.unlink(cr._cache_path)
        os.rmdir(cachedir)

        cr = CrossRef(cachedir='~/testcachedir')
        assert os.path.exists(os.path.expanduser('~/testcachedir'))
        assert cr._cache_path is not None

        os.unlink(cr._cache_path)
        os.rmdir(os.path.expanduser('~/testcachedir'))




import unittest

from metapub.convert import pmid2doi, pmid2doi_with_score, PubMedArticle2doi, PubMedArticle2doi_with_score

pmid_with_doi_in_PMA = 25847151
pmid_with_doi_in_PMA_expected_doi = "10.1016/j.neulet.2015.04.001"

pmid_with_doi_from_CrossRef = 11228145
pmid_with_doi_from_CrossRef_expected_doi = '10.1126/science.1057766'

pmid_with_unknown_doi = 19634325

class TestConversions(unittest.TestCase):

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_pmid2doi(self):
        #pmid2doi references PubMedArticle2doi, so let's consider that function 
        #implicitly tested here.
        doi = pmid2doi(pmid_with_doi_in_PMA)
        assert doi == pmid_with_doi_in_PMA_expected_doi

        doi = pmid2doi(pmid_with_doi_from_CrossRef)
        assert doi == pmid_with_doi_from_CrossRef_expected_doi

        doi = pmid2doi(pmid_with_unknown_doi)
        assert doi is None

    def test_pmid2doi_with_score(self):
        doi, score = pmid2doi_with_score(pmid_with_doi_in_PMA)
        assert doi == pmid_with_doi_in_PMA_expected_doi
        assert score == 10.0

        doi, score = pmid2doi_with_score(pmid_with_doi_from_CrossRef)
        assert doi == pmid_with_doi_from_CrossRef_expected_doi
        assert score > 2.0

        doi, score = pmid2doi_with_score(pmid_with_unknown_doi)
        assert doi is None
        assert score < 2.0





JOURNAL_CANTDO_LIST = [
    'Nat Monspel Ser Bot',
    'Nat Prod Lett',
    'Nat Hist',
    'Nat Mus',
    'Nat Syst'
    'Nat Resour Forum',
    'Nat Law Forum',
    'Nat Resour J',
    'Zhonghua Yi Xue Yi Chuan Xue Za Zhi',
]




###
#
# This file contains definitions for special methods of 
# forming advanced queries according to the specs on the
# following page:
#
# http://www.ncbi.nlm.nih.gov/books/NBK3827/#pubmedhelp.Clinical_Queries_Filters
#

clinical_query_map = { 

    'therapy_broad': '((clinical[Title/Abstract] AND trial[Title/Abstract]) OR clinical trials as topic[MeSH Terms] OR clinical trial[Publication Type] OR random*[Title/Abstract] OR random allocation[MeSH Terms] OR therapeutic use[MeSH Subheading])',
    'therapy_narrow': '(randomized controlled trial[Publication Type] OR (randomized[Title/Abstract] AND controlled[Title/Abstract] AND trial[Title/Abstract]))',
    'diagnosis_broad': '(sensitiv*[Title/Abstract] OR sensitivity and specificity[MeSH Terms] OR diagnose[Title/Abstract] OR diagnosed[Title/Abstract] OR diagnoses[Title/Abstract] OR diagnosing[Title/Abstract] OR diagnosis[Title/Abstract] OR diagnostic[Title/Abstract] OR diagnosis[MeSH:noexp] OR diagnostic * [MeSH:noexp] OR diagnosis,differential[MeSH:noexp] OR diagnosis[Subheading:noexp])',
    'diagnosis_narrow': '(specificity[Title/Abstract])',
    'etiology_broad': '(risk*[Title/Abstract] OR risk*[MeSH:noexp] OR risk *[MeSH:noexp] OR cohort studies[MeSH Terms] OR group[Text Word] OR groups[Text Word] OR grouped [Text Word])',
    'etiology_narrow': '((relative[Title/Abstract] AND risk*[Title/Abstract]) OR (relative risk[Text Word]) OR risks[Text Word] OR cohort studies[MeSH:noexp] OR (cohort[Title/Abstract] AND study[Title/Abstract]) OR (cohort[Title/Abstract] AND studies[Title/Abstract]))',
    'prognosis_broad': '(incidence[MeSH:noexp] OR mortality[MeSH Terms] OR follow up studies[MeSH:noexp] OR prognos*[Text Word] OR predict*[Text Word] OR course*[Text Word])',
    'prognosis_narrow': '(prognos*[Title/Abstract] OR (first[Title/Abstract] AND episode[Title/Abstract]) OR cohort[Title/Abstract])',
    'prediction_broad': '(predict*[tiab] OR predictive value of tests[mh] OR score[tiab] OR scores[tiab] OR scoring system[tiab] OR scoring systems[tiab] OR observ*[tiab] OR observer variation[mh])',
    'prediction_narrow': '(validation[tiab] OR validate[tiab])',
}


medical_genetics_query_map = {
    'diagnosis': '(Diagnosis AND Genetics)',
    'differential_diagnosis': '(Differential Diagnosis[MeSH] OR Differential Diagnosis[Text Word] AND Genetics)',
    'clinical_description': '(Natural History OR Mortality OR Phenotype OR Prevalence OR Penetrance AND Genetics)',
    'management': '(therapy[Subheading] OR treatment[Text Word] OR treatment outcome OR investigational therapies AND Genetics)',
    'genetic_counseling': '(Genetic Counseling OR Inheritance pattern AND genetics)',
    'molecular_genetics': '(Medical Genetics OR genotype OR genetics[Subheading] AND genetics)',
    'genetic_testing': '(DNA Mutational Analysis OR Laboratory techniques and procedures OR Genetic Markers OR diagnosis OR testing OR test OR screening OR mutagenicity tests OR genetic techniques OR molecular diagnostic techniques AND genetics)',
    'all': '((Diagnosis AND genetics) OR (Differential Diagnosis[MeSH] OR Differential Diagnosis[Text Word] AND genetics) OR (Natural History OR Mortality OR Phenotype OR Prevalence OR Penetrance AND genetics) OR (therapy[Subheading] OR treatment[Text Word] OR treatment outcome OR investigational therapies AND genetics) OR (Genetic Counseling OR Inheritance pattern AND genetics) OR (Medical Genetics OR genotype OR genetics[Subheading] AND genetics) OR (DNA Mutational Analysis OR Laboratory techniques and procedures OR Genetic Markers OR diagnosis OR testing OR test OR screening OR mutagenicity tests OR genetic techniques OR molecular diagnostic techniques AND genetics))',
}





import unittest
from metapub.exceptions import *
from metapub import PubMedArticle, PubMedFetcher

import random

xml_str1 = '''<?xml version="1.0"?>
<!DOCTYPE PubmedArticleSet PUBLIC "-//NLM//DTD PubMedArticle, 1st January 2015//EN" "http://www.ncbi.nlm.nih.gov/corehtml/query/DTD/pubmed_150101.dtd">
<PubmedArticleSet>
<PubmedArticle>
    <MedlineCitation Owner="NLM" Status="MEDLINE">
        <PMID Version="1">4</PMID>
        <DateCreated>
            <Year>1976</Year>
            <Month>01</Month>
            <Day>10</Day>
        </DateCreated>
        <DateCompleted>
            <Year>1976</Year>
            <Month>01</Month>
            <Day>10</Day>
        </DateCompleted>
        <DateRevised>
            <Year>2013</Year>
            <Month>11</Month>
            <Day>21</Day>
        </DateRevised>
        <Article PubModel="Print">
            <Journal>
                <ISSN IssnType="Print">0006-291X</ISSN>
                <JournalIssue CitedMedium="Print">
                    <Volume>66</Volume>
                    <Issue>4</Issue>
                    <PubDate>
                        <Year>1975</Year>
                        <Month>Oct</Month>
                        <Day>27</Day>
                    </PubDate>
                </JournalIssue>
                <Title>Biochemical and biophysical research communications</Title>
                <ISOAbbreviation>Biochem. Biophys. Res. Commun.</ISOAbbreviation>
            </Journal>
            <ArticleTitle>Effect of chloroquine on cultured fibroblasts: release of lysosomal hydrolases and inhibition of their uptake.</ArticleTitle>
            <Pagination>
                <MedlinePgn>1338-43</MedlinePgn>
            </Pagination>
            <AuthorList CompleteYN="Y">
                <Author ValidYN="Y">
                    <LastName>Wiesmann</LastName>
                    <ForeName>U N</ForeName>
                    <Initials>UN</Initials>
                </Author>
                <Author ValidYN="Y">
                    <LastName>DiDonato</LastName>
                    <ForeName>S</ForeName>
                    <Initials>S</Initials>
                </Author>
                <Author ValidYN="Y">
                    <LastName>Herschkowitz</LastName>
                    <ForeName>N N</ForeName>
                    <Initials>NN</Initials>
                </Author>
            </AuthorList>
            <Language>eng</Language>
            <PublicationTypeList>
                <PublicationType UI="D016428">Journal Article</PublicationType>
            </PublicationTypeList>
        </Article>
        <MedlineJournalInfo>
            <Country>UNITED STATES</Country>
            <MedlineTA>Biochem Biophys Res Commun</MedlineTA>
            <NlmUniqueID>0372516</NlmUniqueID>
            <ISSNLinking>0006-291X</ISSNLinking>
        </MedlineJournalInfo>
        <ChemicalList>
            <Chemical>
                <RegistryNumber>886U3H6UFF</RegistryNumber>
                <NameOfSubstance UI="D002738">Chloroquine</NameOfSubstance>
            </Chemical>
            <Chemical>
                <RegistryNumber>EC 3.1.6.-</RegistryNumber>
                <NameOfSubstance UI="D013429">Sulfatases</NameOfSubstance>
            </Chemical>
            <Chemical>
                <RegistryNumber>EC 3.1.6.8</RegistryNumber>
                <NameOfSubstance UI="D002553">Cerebroside-Sulfatase</NameOfSubstance>
            </Chemical>
            <Chemical>
                <RegistryNumber>EC 3.2.1.31</RegistryNumber>
                <NameOfSubstance UI="D005966">Glucuronidase</NameOfSubstance>
            </Chemical>
            <Chemical>
                <RegistryNumber>K3R6ZDH4DU</RegistryNumber>
                <NameOfSubstance UI="D003911">Dextrans</NameOfSubstance>
            </Chemical>
        </ChemicalList>
        <CitationSubset>IM</CitationSubset>
        <MeshHeadingList>
            <MeshHeading>
                <DescriptorName MajorTopicYN="N" UI="D001692">Biological Transport</DescriptorName>
            </MeshHeading>
            <MeshHeading>
                <DescriptorName MajorTopicYN="N" UI="D002478">Cells, Cultured</DescriptorName>
            </MeshHeading>
            <MeshHeading>
                <DescriptorName MajorTopicYN="N" UI="D002553">Cerebroside-Sulfatase</DescriptorName>
                <QualifierName MajorTopicYN="Y" UI="Q000378">metabolism</QualifierName>
            </MeshHeading>
            <MeshHeading>
                <DescriptorName MajorTopicYN="N" UI="D002738">Chloroquine</DescriptorName>
                <QualifierName MajorTopicYN="Y" UI="Q000494">pharmacology</QualifierName>
            </MeshHeading>
            <MeshHeading>
                <DescriptorName MajorTopicYN="N" UI="D003911">Dextrans</DescriptorName>
                <QualifierName MajorTopicYN="N" UI="Q000378">metabolism</QualifierName>
            </MeshHeading>
            <MeshHeading>
                <DescriptorName MajorTopicYN="N" UI="D005347">Fibroblasts</DescriptorName>
                <QualifierName MajorTopicYN="N" UI="Q000201">enzymology</QualifierName>
                <QualifierName MajorTopicYN="N" UI="Q000378">metabolism</QualifierName>
            </MeshHeading>
            <MeshHeading>
                <DescriptorName MajorTopicYN="N" UI="D005966">Glucuronidase</DescriptorName>
                <QualifierName MajorTopicYN="Y" UI="Q000378">metabolism</QualifierName>
            </MeshHeading>
            <MeshHeading>
                <DescriptorName MajorTopicYN="N" UI="D006801">Humans</DescriptorName>
            </MeshHeading>
            <MeshHeading>
                <DescriptorName MajorTopicYN="N" UI="D007966">Leukodystrophy, Metachromatic</DescriptorName>
                <QualifierName MajorTopicYN="N" UI="Q000201">enzymology</QualifierName>
            </MeshHeading>
            <MeshHeading>
                <DescriptorName MajorTopicYN="N" UI="D008247">Lysosomes</DescriptorName>
                <QualifierName MajorTopicYN="N" UI="Q000187">drug effects</QualifierName>
                <QualifierName MajorTopicYN="Y" UI="Q000201">enzymology</QualifierName>
            </MeshHeading>
            <MeshHeading>
                <DescriptorName MajorTopicYN="N" UI="D010873">Pinocytosis</DescriptorName>
                <QualifierName MajorTopicYN="N" UI="Q000187">drug effects</QualifierName>
            </MeshHeading>
            <MeshHeading>
                <DescriptorName MajorTopicYN="N" UI="D012867">Skin</DescriptorName>
                <QualifierName MajorTopicYN="N" UI="Q000201">enzymology</QualifierName>
            </MeshHeading>
            <MeshHeading>
                <DescriptorName MajorTopicYN="N" UI="D013429">Sulfatases</DescriptorName>
                <QualifierName MajorTopicYN="Y" UI="Q000378">metabolism</QualifierName>
            </MeshHeading>
        </MeshHeadingList>
    </MedlineCitation>
    <PubmedData>
        <History>
            <PubMedPubDate PubStatus="pubmed">
                <Year>1975</Year>
                <Month>10</Month>
                <Day>27</Day>
            </PubMedPubDate>
            <PubMedPubDate PubStatus="medline">
                <Year>1975</Year>
                <Month>10</Month>
                <Day>27</Day>
                <Hour>0</Hour>
                <Minute>1</Minute>
            </PubMedPubDate>
            <PubMedPubDate PubStatus="entrez">
                <Year>1975</Year>
                <Month>10</Month>
                <Day>27</Day>
                <Hour>0</Hour>
                <Minute>0</Minute>
            </PubMedPubDate>
        </History>
        <PublicationStatus>ppublish</PublicationStatus>
        <ArticleIdList>
            <ArticleId IdType="pubmed">4</ArticleId>
            <ArticleId IdType="pii">0006-291X(75)90506-9</ArticleId>
        </ArticleIdList>
    </PubmedData>
</PubmedArticle>

</PubmedArticleSet>
'''

xml_str2 = '''
    <MedlineCitation Owner="NLM" Status="MEDLINE">
        <PMID Version="1">23697015</PMID>
        <DateCreated>
            <Year>2013</Year>
            <Month>05</Month>
            <Day>23</Day>
        </DateCreated>
        <DateCompleted>
            <Year>2013</Year>
            <Month>07</Month>
            <Day>11</Day>
        </DateCompleted>
        <Article PubModel="Print">
        <Journal>
            <ISSN IssnType="Print">1110-0583</ISSN>
            <JournalIssue CitedMedium="Print">
                <Volume>43</Volume>
                <Issue>1</Issue>
                <PubDate>
                    <Year>2013</Year>
                    <Month>Apr</Month>
                </PubDate>
            </JournalIssue>
            <Title>Journal of the Egyptian Society of Parasitology</Title>
            <ISOAbbreviation>J Egypt Soc Parasitol</ISOAbbreviation>
        </Journal>
        <ArticleTitle>The Rift Valley fever: could re-emerge in Egypt again?</ArticleTitle>
        <Pagination>
            <MedlinePgn>41-56</MedlinePgn>
        </Pagination>
        <Abstract>
            <AbstractText>The Rift Valley fever (RVF) is a neglected, emerging, mosquito-borne disease with severe negative impact on human and animal health and economy. RVF is caused by RVF virus of the family of Bunyaviridae, genus Phlebovirus. RVF is an acute, febrile disease affecting humans and a wide range of animals. The virus is trans-mitted through the bites from mosquitoes and exposure to viremic blood, body fluids, or contact with tissues of infected animals or by inhaling natural virus aerosols, also possibly by consumption of infected unpasteurized milk. The RVF-virus replicate at the site introduction and in local lymphatic followed by viremia and spread to other organs as the liver and central nervous system, causing the hepatic necrosis and eosinophilia cytoplasmic degeneration. The main signs and symptoms are fever, headache, myalgia, arthralgia, photophobia, bradycardia, conjunctivitis and flushing face. Main complications include jaundice, hemorrhagic, meningoencephalitis and retinal lesions. Generally speaking, in the 21st Century, the vector-borne infectious diseases, was accepted as the disaster issues with the considerable significant morbidity and mortality. These facts should be considered by the public health, veterinary and agricultural authorities</AbstractText>
        </Abstract>
        <AuthorList CompleteYN="Y">
            <Author ValidYN="Y">
                <LastName>El-Bahnasawy</LastName>
                <ForeName>Mamdouh</ForeName>
                <Initials>M</Initials>
                <Affiliation>The Military Medical Services For Preventive Medicine, Ain Shams University, Cairo 11566, Egypt.</Affiliation>
            </Author>
            <Author ValidYN="Y">
                <LastName>Megahed</LastName>
                <ForeName>Laila Abdel-Mawla</ForeName>
                <Initials>LA</Initials>
            </Author>
            <Author ValidYN="Y">
                <LastName>Abdalla Saleh</LastName>
                <ForeName>Hala Ahmed</ForeName>
                <Initials>HA</Initials>
            </Author>
            <Author ValidYN="Y">
                <LastName>Morsy</LastName>
                <ForeName>Tosson A</ForeName>
                <Initials>TA</Initials>
            </Author>
        </AuthorList>
        <Language>eng</Language>
        <PublicationTypeList>
            <PublicationType>Journal Article</PublicationType>
        </PublicationTypeList>
    </Article>
    <MedlineJournalInfo>
        <Country>Egypt</Country>
        <MedlineTA>J Egypt Soc Parasitol</MedlineTA>
        <NlmUniqueID>8102141</NlmUniqueID>
        <ISSNLinking>1110-0583</ISSNLinking>
    </MedlineJournalInfo>
    <CitationSubset>IM</CitationSubset>
    <MeshHeadingList>
        <MeshHeading>
            <DescriptorName MajorTopicYN="N">Animals</DescriptorName>
        </MeshHeading>
        <MeshHeading>
            <DescriptorName MajorTopicYN="N">Communicable Diseases, Emerging</DescriptorName>
            <QualifierName MajorTopicYN="Y">epidemiology</QualifierName>
        </MeshHeading>
        <MeshHeading>
            <DescriptorName MajorTopicYN="N">Culicidae</DescriptorName>
        </MeshHeading>
        <MeshHeading>
            <DescriptorName MajorTopicYN="N" Type="Geographic">Egypt</DescriptorName>
            <QualifierName MajorTopicYN="N">epidemiology</QualifierName>
        </MeshHeading>
        <MeshHeading>
            <DescriptorName MajorTopicYN="N">Humans</DescriptorName>
        </MeshHeading>
        <MeshHeading>
            <DescriptorName MajorTopicYN="N">Insect Vectors</DescriptorName>
        </MeshHeading>
        <MeshHeading>
            <DescriptorName MajorTopicYN="N">Phlebovirus</DescriptorName>
        </MeshHeading>
        <MeshHeading>
            <DescriptorName MajorTopicYN="N">Rift Valley Fever</DescriptorName>
            <QualifierName MajorTopicYN="Y">epidemiology</QualifierName>
        </MeshHeading>
        <MeshHeading>
            <DescriptorName MajorTopicYN="N" Type="Geographic">Saudi Arabia</DescriptorName>
            <QualifierName MajorTopicYN="N">epidemiology</QualifierName>
        </MeshHeading>
        <MeshHeading>
            <DescriptorName MajorTopicYN="N" Type="Geographic">Sudan</DescriptorName>
            <QualifierName MajorTopicYN="N">epidemiology</QualifierName>
        </MeshHeading>
    </MeshHeadingList>
</MedlineCitation>
'''


class TestPubMedArticle(unittest.TestCase):

    def setUp(self):
        self.fetch = PubMedFetcher()

    def tearDown(self):
        pass

    def test_random_efetch(self):
        pmid = str(random.randint(22222222, 23333333))
        try:
            article = self.fetch.article_by_pmid(pmid)
            assert article.pmid == pmid
            assert article.title is not None
        except InvalidPMID, e:
            print "PMID %i returned InvalidPMID response (which is totally OK). Run test again!" % pmid
        

    def test_init1(self):
        '''
        Test on the xml returned by eutils
        '''
        article = PubMedArticle(xml_str1)
        assert str(article.pmid) == '4'

    def test_init2(self):
        '''
        Test on the xml downloaded from medline
        '''
        article = PubMedArticle(xml_str2)
        assert str(article.pmid) == '23697015'

